@@ -21,6 +21,7 @@ import com.unciv.models.ruleset.unique.Unique
 import com.unciv.models.ruleset.unique.UniqueTarget
 import com.unciv.models.ruleset.unique.UniqueTriggerActivation
 import com.unciv.models.ruleset.unique.UniqueType
+import com.unciv.models.translations.fillPlaceholders
 import com.unciv.models.translations.removeConditionals
 import com.unciv.models.translations.tr
 import com.unciv.ui.components.Fonts
@@ -158,12 +159,8 @@ object UnitActions {
         return UnitAction(UnitActionType.Create, "Create [$improvementName]",
             action = {
                 tile.changeImprovement(improvementName)
-                val city = tile.getCity()
-                if (city != null) {
-                    city.cityStats.update()
-                    city.civ.cache.updateCivResources()
-                }
-                unit.destroy()
+                tile.getTileImprovement()!!.handleImprovementCompletion(unit)
+                unit.destroy()  // Modders may wish for a nondestructive way, but that should be another Unique
             }.takeIf { unit.currentMovement > 0 })
     }
 
@@ -204,6 +201,7 @@ object UnitActions {
 
             if (hasActionModifiers) activateSideEffects(unit, unique)
             else unit.destroy()
+            GUI.setUpdateWorldOnNextRender() // Set manually, since this could be triggered from the ConfirmPopup and not from the UnitActionsTable
         }
 
         if (unit.civ.playerType == PlayerType.AI)
@@ -329,30 +327,32 @@ object UnitActions {
         val transformList = ArrayList<UnitAction>()
         for (unique in unit.baseUnit().getMatchingUniques(UniqueType.CanTransform,
             StateForConditionals(unit = unit, civInfo = civInfo, tile = unitTile))) {
-            val upgradedUnit = civInfo.getEquivalentUnit(unique.params[0])
-            // don't show if haven't researched/is obsolete
-            if (!unit.upgrade.canUpgrade(unitToUpgradeTo = upgradedUnit)) continue
+            val unitToTransformTo = civInfo.getEquivalentUnit(unique.params[0])
+
+            val stateForConditionals = StateForConditionals(unit.civ, unit = unit)
+            if (unitToTransformTo.getMatchingUniques(UniqueType.OnlyAvailableWhen).any { !it.conditionalsApply(stateForConditionals) })
+                continue
 
             // Check _new_ resource requirements
             // Using Counter to aggregate is a bit exaggerated, but - respect the mad modder.
             val resourceRequirementsDelta = Counter<String>()
             for ((resource, amount) in unit.baseUnit().getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, -amount)
-            for ((resource, amount) in upgradedUnit.getResourceRequirementsPerTurn())
+            for ((resource, amount) in unitToTransformTo.getResourceRequirementsPerTurn())
                 resourceRequirementsDelta.add(resource, amount)
             val newResourceRequirementsString = resourceRequirementsDelta.entries
                 .filter { it.value > 0 }
                 .joinToString { "${it.value} {${it.key}}".tr() }
 
             val title = if (newResourceRequirementsString.isEmpty())
-                "Transform to [${upgradedUnit.name}]"
-            else "Transform to [${upgradedUnit.name}]\n([$newResourceRequirementsString])"
+                "Transform to [${unitToTransformTo.name}]"
+            else "Transform to [${unitToTransformTo.name}]\n([$newResourceRequirementsString])"
 
             transformList.add(UnitAction(UnitActionType.Transform,
                 title = title,
                 action = {
                     unit.destroy()
-                    val newUnit = civInfo.units.placeUnitNearTile(unitTile.position, upgradedUnit.name)
+                    val newUnit = civInfo.units.placeUnitNearTile(unitTile.position, unitToTransformTo.name)
 
                     /** We were UNABLE to place the new unit, which means that the unit failed to upgrade!
                      * The only known cause of this currently is "land units upgrading to water units" which fail to be placed.
@@ -365,9 +365,7 @@ object UnitActions {
                         newUnit.currentMovement = 0f
                     }
                 }.takeIf {
-                    unit.currentMovement > 0
-                            && !unit.isEmbarked()
-                            && unit.upgrade.canUpgrade(unitToUpgradeTo = upgradedUnit)
+                    unit.currentMovement > 0 && !unit.isEmbarked()
                 }
             ) )
         }
@@ -501,7 +499,7 @@ object UnitActions {
                     unitTile.stopWorkingOnImprovement()
                     improvement.handleImprovementCompletion(unit)
 
-                    // without this the world screen won't the improvement because it isn't the 'last seen improvement'
+                    // without this the world screen won't show the improvement because it isn't the 'last seen improvement'
                     unit.civ.cache.updateViewableTiles()
 
                     if (unique.type == UniqueType.ConstructImprovementConsumingUnit) unit.consume()
@@ -675,9 +673,14 @@ object UnitActions {
                     && unique.conditionals.none { it.type == UniqueType.ConditionalTimedUnique }) continue
             if (usagesLeft(unit, unique)==0) continue
 
-            val unitAction = UnitAction(type = UnitActionType.TriggerUnique,
-                title = actionTextWithSideEffects(unique.text.removeConditionals(), unique, unit)
-                ){
+            val baseTitle = if (unique.isOfType(UniqueType.OneTimeEnterGoldenAgeTurns))
+                    unique.placeholderText.fillPlaceholders(
+                        unit.civ.goldenAges.calculateGoldenAgeLength(
+                            unique.params[0].toInt()).toString())
+                else unique.text.removeConditionals()
+            val title = actionTextWithSideEffects(baseTitle, unique, unit)
+
+            val unitAction = UnitAction(type = UnitActionType.TriggerUnique, title){
                 UniqueTriggerActivation.triggerUnitwideUnique(unique, unit)
                 activateSideEffects(unit, unique)
             }
@@ -758,7 +761,7 @@ object UnitActions {
         return null
     }
 
-    fun actionTextWithSideEffects(originalText:String, actionUnique: Unique, unit: MapUnit): String {
+    fun actionTextWithSideEffects(originalText: String, actionUnique: Unique, unit: MapUnit): String {
         val sideEffectString = getSideEffectString(unit, actionUnique)
         if (sideEffectString == "") return originalText
         else return "{$originalText} $sideEffectString"
